package no.statnett.ecp;

import no.statnett.ecp.brs.BrokerShell;
import no.statnett.ecp.div.BrannslukkerAFRRCAPBudTilNVE;
import no.statnett.ecp.div.SC;
import no.statnett.ecp.emd.ECPMsgDelay;
import no.statnett.ecp.eml.ECPMsgLogger;
import no.statnett.ecp.epa.ECPPerfAnalyzer;
import no.statnett.ecp.gm.GapMonitor;
import no.statnett.ecp.qc.QueueCleaner;
import no.statnett.ecp.qp.QueueProtector;
import no.statnett.ecp.utils.Div;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.LogManager;

public class EKit {

    public static final String VERSION = "v1.6.1";

    public static void main(String[] args) throws Exception {
        LogManager.getLogManager().getLogger("").setLevel(Level.OFF); // Mutes unwanted log-messages from the libraries/dependencies
        if (args.length < 1) {
            usage();
        } else {
            try {
                List<String> argsList = Arrays.asList(args);
                String tool = argsList.get(0);
                // remove first element of argsList
                argsList = new ArrayList<>(argsList.subList(1, argsList.size()));


                args = argsList.toArray(new String[argsList.size()]);
                if (tool.equals("QC")) {
                    QueueCleaner.main(args);
                } else if (tool.equals("QP")) {
                    QueueProtector.main(args);
                } else if (tool.equals("EMD")) {
                    ECPMsgDelay.main(args);
                } else if (tool.equals("EPA")) {
                    ECPPerfAnalyzer.main(args);
                } else if (tool.equals("EML")) {
                    ECPMsgLogger.main(args);
                } else if (tool.equals("GM")) {
                    GapMonitor.main(args);
                } else if (tool.equals("BRS")) {
                    BrokerShell.main(args);
                } else if (tool.equals("SC")) {
                    SC.main(args); // non-exposed tools, only useful for dealing with EDX Service Catalogue - old syntax - little interest
                } else if (tool.equals("ACNVE")) {
                    BrannslukkerAFRRCAPBudTilNVE.main(args); // non-exposed, very specific tool for Statnett (retrieves some files from EDX-DMS if they can be found, filters XML and send to NVE)
                } else {
                    usage();
                }
            } catch (Exception e) {
                System.err.println("Error occurred: " + e + ", Error-message: " + e.getMessage() + ", Stacktrace: " + Div.onelinerStacktrace(e.getStackTrace(), "no.statnett.ecp"));
            }
        }
    }

    private static void usage() {
        System.out.println("\nekit " + VERSION + " is short for ECCoSP Kit, built by Statnett (Morten Simonsen) for ECP 4.8-4.14 (EDX v1.9-1.14)");
        System.out.println("The kit offers the following\n");
        System.out.println("1) QueueCleaner (QC) " + QueueCleaner.VERSION + " - monitor all queues in ECP/EDX, remove messages based on filters");
        System.out.println("2) QueueProtector (QP) " + QueueProtector.VERSION + " - protect an ECP/EDX queue from flooding/filling; can also restore messages");
        System.out.println("3) ECPMsgDelay (EMD) " + ECPMsgDelay.VERSION + " - analyzes ECP-logs to calculate delay for messages transmitted over the ECP-network");
        System.out.println("4) ECPPerfAnalyzer (EPA) " + ECPPerfAnalyzer.VERSION + " - analyzes ECP/EDX-logs to calculate throughput of the EPC/EDX-endpoint");
        System.out.println("5) ECPMsgLogger (EML) " + ECPMsgLogger.VERSION + " - makes a logline for each ECP-message in/out, useful for monitoring and analysis");
        System.out.println("6) GapMonitor (GM) " + GapMonitor.VERSION + " - monitor gaps in the ECP-message traffic - specific for entry-criteria to NBM");
        System.out.println("7) BrokerShell (BRS) " + BrokerShell.VERSION + " - manage/monitor Artemis-brokers (ECP 4.14+)");
        System.out.println("\nTo get more help on usage, run 'java -jar ekit.jar <QC|QP|EMD|EPA|EML|GM|BRS>'");
        System.out.println("\n\nDisclaimer/License: ekit.jar can be used by anyone for any purpose without any restrictions. The tools are used in production ");
        System.out.println("in Statnett, but the author does not guarantee that the tools are bug-free or that they work as expected - no liability is taken. ");
        System.out.println("The source code is attached to the jar-file and can be viewed/scanned. The only major library used outside of core Java-classes is ");
        System.out.println("QPid JMS which is necessary to connect to AMQP-broker. The Qpid library in turn depends upon Netty library. Apart from this all ");
        System.out.println("other code is written by the author. Another goal for this code is that it should have a small footprint when it comes to CPU/Memory.");
        System.out.println();
    }


}
