package no.statnett.ecp.utils;

import javax.net.ssl.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.Base64;

public class ArtemisConsoleAPI {

    public static void main(String[] args) throws IOException, NoSuchAlgorithmException, KeyManagementException {
        String username = "endpoint";
        String password = "password";
        String protocol = "https";
        String host = "h1-a-ecp-apt01.statnett.no";

        String port = "8161";
//        String queueName = "ecp.endpoint.upload.45V0000000000318";

        System.out.println("Broker-name: " + retrieveBrokerNameURLEncoded(protocol, host, port, username, password));
//        printQueueInfo(host, port, brokerName, username, password);
//        printRemovedMessages(host, port, brokerName, username, password, queueName);
    }


    // Broker-list QueryString /console/jolokia/search/org.apache.activemq.artemis:broker=*"
    public static String retrieveBrokerNameURLEncoded(String protocol, String host, String port, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/search/org.apache.activemq.artemis:broker=*");
        String json = getJSON(url, username, password);
        // Example of json: {"request":{"mbean":"org.apache.activemq.artemis:broker=*","type":"search"},"value":["org.apache.activemq.artemis:broker=\"ECCo SP Artemis\""],"timestamp":1720632774,"status":200}
        // Extract broker-name from json
        int valuesPos = json.indexOf("value\":") + 9;
        int brokerNameStartPos = json.indexOf("broker=", valuesPos) + 7;
        int brokerNameEndPos = json.indexOf("]", brokerNameStartPos);
        String brokerNameWithQuotes = json.substring(brokerNameStartPos, brokerNameEndPos);
        String brokerName = brokerNameWithQuotes.replaceAll("\"", "").replaceAll("\\\\", "");
        return brokerName.replaceAll(" ", "%20");
    }

    // Broker-info QueryString  /console/jolokia/read/org.apache.activemq.artemis:broker=%22ECCo%20SP%20Artemis%22
    public static String retrieveBrokerInfoJSON(String protocol, String host, String port, String brokerNameEncoded, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/read/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22");
        return getJSON(url, username, password);
    }

    // Queue-info QueryString:  /console/jolokia/exec/org.apache.activemq.artemis:broker=%22ECCo%20SP%20Artemis%22/listQueues/%7B%22field%22:%22name%22%2C%22operation%22:%22CONTAINS%22%2C%22value%22:%22ecp%22%7D/1/1000
    public static String retrieveQueueInfoJSON(String protocol, String host, String port, String brokerNameEncoded, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/exec/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22/listQueues/%7B%22field%22:%22name%22%2C%22operation%22:%22NOT%5FCONTAINS%22%2C%22value%22:%22abcdefghijkl%22%7D/1/1000");
        return getJSON(url, username, password);
    }

    private static String findAttributeValue(String json, String name) {
        int statusStartPos = json.indexOf(name + "\":") + name.length() + 2;
        if (statusStartPos <= (-1 + name.length() + 2))
            return null;
        int statusEndPos = json.indexOf(",", statusStartPos) == -1 ? json.indexOf("}", statusStartPos) : json.indexOf(",", statusStartPos);
        return json.substring(statusStartPos, statusEndPos);
    }

    // RemoveMessages QueryString: /console/jolokia/exec/org.apache.activemq.artemis:broker=%22ECCo%20SP%20Artemis%22%2Ccomponent=addresses%2Caddress=%22ecp.endpoint.upload.45V0000000000318%22%2Csubcomponent=queues%2Crouting-type=%22anycast%22%2Cqueue=%22ecp.endpoint.upload.45V0000000000318%22/removeAllMessages
    public static ACAPIResult purgeQueue(String protocol, String host, String port, String brokerNameEncoded, String username, String password, String queueName) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        if (queueName.contains("://")) {
            queueName = queueName.replace("://", "%3A%2F%2F");
        }
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/exec/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22%2Ccomponent=addresses%2Caddress=%22" + queueName + "%22%2Csubcomponent=queues%2Crouting-type=%22anycast%22%2Cqueue=%22" + queueName + "%22/removeAllMessages");
        String json = getJSON(url, username, password);
        String status = findAttributeValue(json, "status");
        String value = findAttributeValue(json, "value");
        return new ACAPIResult(status, value, json);
    }

    public static boolean deleteQueue(String protocol, String host, String port, String brokerNameEncoded, String username, String password, String queueName) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        // Keep in reserve: %2Ccomponent=addresses%2Caddress=%22" + queueName + "%22%2Csubcomponent=queues%2Crouting-type=%22anycast%22%2Cqueue=%22" + queueName + "%22
        URL url = new URL(protocol + "://" + host + ":" + port + "/console/jolokia/exec/org.apache.activemq.artemis:broker=%22" + brokerNameEncoded + "%22/destroyQueue(java.lang.String)/" + queueName);
        String json = getJSON(url, username, password);
        String status = findAttributeValue(json, "status");
        if (status.equals("200"))
            return true;
        else
            return false;
    }

    private static String getJSON(URL url, String username, String password) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        BufferedReader bufferedReader = null;
        try {
            if (url.getProtocol().equals("https")) {
                // Step 2: Set up a TrustManager that trusts all certificates
                TrustManager[] trustAll = new TrustManager[]{
                        new X509TrustManager() {
                            @Override
                            public X509Certificate[] getAcceptedIssuers() {
                                return null;
                            }

                            @Override
                            public void checkClientTrusted(X509Certificate[] certs, String authType) {
                            }

                            @Override
                            public void checkServerTrusted(X509Certificate[] certs, String authType) {
                            }
                        }
                };
                SSLContext sslContext = SSLContext.getInstance("TLS");
                sslContext.init(null, trustAll, null);

                HttpsURLConnection.setDefaultSSLSocketFactory(sslContext.getSocketFactory());

                // Step 4: Set up a HostnameVerifier that accepts all hostnames
                HostnameVerifier allHostsValid = (hostname, session) -> true;
                HttpsURLConnection.setDefaultHostnameVerifier(allHostsValid);

                // Step 5: Open a connection
                HttpsURLConnection connection = (HttpsURLConnection) url.openConnection();

                connection.setRequestMethod("GET");
                connection.setRequestProperty("Accept", "application/json");

                // Step 6: Add Basic Authentication
                String auth = username + ":" + password;
                String encodedAuth = Base64.getEncoder().encodeToString(auth.getBytes());
                String authHeader = "Basic " + encodedAuth;
                connection.setRequestProperty("Authorization", authHeader);
                connection.connect();
                bufferedReader = new BufferedReader(new InputStreamReader(connection.getResponseCode() == 200 ? connection.getInputStream() : connection.getErrorStream(), StandardCharsets.UTF_8));
            } else {
                // Step 5: Open a connection
                HttpURLConnection connection = (HttpURLConnection) url.openConnection();

                connection.setRequestMethod("GET");
                connection.setRequestProperty("Accept", "application/json");

                // Step 6: Add Basic Authentication
                String auth = username + ":" + password;
                String encodedAuth = Base64.getEncoder().encodeToString(auth.getBytes());
                String authHeader = "Basic " + encodedAuth;
                connection.setRequestProperty("Authorization", authHeader);
                connection.connect();
                bufferedReader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            }
        } catch (Exception e) {
            System.out.println("\nError occurred: Connection to Artemis-broker on URL: " + url + " using user/pass " + username + "/" + password + " failed - we expect Artemis-console to be available on HTTPS on port 8161 (hardcoded). Check user/pass + firewall + jolokia-access.xml to ensure access.");
            throw e;
        }
        if (bufferedReader == null)
            return "";
        String line;
        StringBuilder builder = new StringBuilder();
        while ((line = bufferedReader.readLine()) != null) {
            builder.append(line);
        }
        bufferedReader.close();
        if (builder.toString().contains("error_type")) {
            // We tried to check the connection.getReponseMessage() != 200, but it did not work as expected, so we resort to checking the output
            String errorMessage = "Error occurred: HTTP-response contains errors: " + builder + " for URL: " + url + " using user/pass " + username + "/" + password;
            System.out.println("\n" + errorMessage);
            throw new IOException(errorMessage);
        }
        return builder.toString();

    }

    public static class ACAPIResult {
        private String status;
        private String value;
        private String json;

        public ACAPIResult(String status, String value, String json) {
            this.status = status;
            this.value = value;
            this.json = json;
        }

        public String getStatus() {
            return status;
        }

        public String getValue() {
            return value;
        }

        public String getJson() {
            return json;
        }
    }
}
