package no.statnett.ecp.utils;

import no.statnett.ecp.eml.ECPMsgLogger;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;

public class DebugLog {

    private static File debugFile;

    public DebugLog(File debugFile) {
        this.debugFile = debugFile;
    }


    public void debug(String message) {
        debug(message, null);
    }

    public void debug(String message, Exception e) {
        try {
            if (debugFile != null) {
                String logLine = LogOut.d() + " " + message + "\n";
                if (e != null) {
                    logLine += LogOut.e() + " " + message + " Exception: " + e.getMessage() + ",  Stacktrace: " + Div.onelinerStacktrace(e.getStackTrace(), "no.statnett.ecp") + "\n";
                }
                if (debugFile.exists()) {
                    if (debugFile.length() > 1000000) {// 1MB (arbitrary limit - will be removed if exceeded
                        Files.delete(debugFile.toPath());
                        Files.writeString(debugFile.toPath(), LogOut.d() + " Deleted previous debug-file since it exceeded 1MB - starting over\n");
                    }
                    Files.writeString(debugFile.toPath(), logLine, StandardOpenOption.APPEND);
                } else
                    Files.writeString(debugFile.toPath(), logLine);
            }
        } catch (IOException ioe) {
            System.err.println("Error occurred during printing to debug-file " + debugFile.getAbsoluteFile() + ": " + ioe.getMessage());
            System.exit(1);
        }
    }
}
